﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RFID.Reader;
using RFID.ReaderComm;

namespace ExampleCodes
{
    class ReadSetAntenna
    {
        static String DEFAULT_READER_IP_ADDRESS = "192.168.1.120";
        static UInt16 PORT_NUMBER = 5000;
        static RFIDReader reader = null;

        public static void Main(String[] args){
            try
            {
                String readerIPAddress = DEFAULT_READER_IP_ADDRESS;
                UInt16 PortNumber = PORT_NUMBER;

                Console.WriteLine("Setting up ReaderComm instance on IP Address " + readerIPAddress);

                reader = new RFIDReader(readerIPAddress, PortNumber);

                Console.WriteLine("Getting Reader Info\n");

                ReadSequence();
                GetAntennaPowerSettings();
                ReadGapTimes();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Main: " + ex.ToString());
            }
        }

        public static void ReadSequence()
        {
            byte[] antennaSequence = new byte[16];
            byte sequenceLength = 0;

            RFIDStatus status = ReadSetAntenna.reader.GetReadSequence(ref antennaSequence, ref sequenceLength);

            Console.WriteLine(String.Format("Antenna sequence (" + sequenceLength + ")"));

            for (int idx = 0; idx < sequenceLength; idx++)
                Console.WriteLine(antennaSequence[idx]);

            Console.WriteLine();

            byte[] newAntSeq = new byte[5];
            newAntSeq[0] = 1; newAntSeq[1] = 0; newAntSeq[2] = 3; newAntSeq[3] = 2; newAntSeq[4] = 1;

            if (RFIDStatus.OK == ReadSetAntenna.reader.SetReadSequence(newAntSeq, (byte)newAntSeq.Length))
                Console.WriteLine("Set antenna sequence succeeded");
            else
                Console.WriteLine("Set antenna sequence failed");

            byte[] validateAntSequence = new byte[16];
            byte validateSequenceLength = 0;

            status = ReadSetAntenna.reader.GetReadSequence(ref validateAntSequence, ref validateSequenceLength);

            if (validateSequenceLength != newAntSeq.Length)
                Console.WriteLine("Validate antenna sequence failed: length didn't match");

            for (int idx = 0; idx < newAntSeq.Length; idx++)
            {
                if (validateAntSequence[idx] != newAntSeq[idx])
                    Console.WriteLine(String.Format("Validate antenna sequence failed seq[" + idx + "] " + newAntSeq[idx] + " != " + validateAntSequence[idx]));

                ReadSetAntenna.reader.SetReadSequence(antennaSequence, sequenceLength);
            }
        }

        public static void GetAntennaPowerSettings()
        {
            Double[] readPower = new Double[4];
            Double[] writePower = new Double[4];

            try
            {
                int numberOfAntennas = 0;
                RFIDStatus status = ReadSetAntenna.reader.GetPowerSetting(out numberOfAntennas, ref readPower, ref writePower);

                for (int idx = 0; idx < numberOfAntennas; idx++)
                {
                    if (status == RFIDStatus.OK)
                    {
                        Console.WriteLine("Antenna " + idx + " read power " + readPower[idx]);
                        Console.WriteLine("Antenna " + idx + " write power " + writePower[idx] + "\n");
                    }
                    else
                        Console.WriteLine("Get Power Settings for antenna failed. Cause unkown.");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Power: " + ex.Message + Environment.NewLine);
            }
        }

        public static void ReadGapTimes()
        {
            UInt16 readTime = 0;
            UInt16 gapTime = 0;

            try
            {

                if (ReadSetAntenna.reader.GetReadGapTimes(ref readTime, ref gapTime) == RFIDStatus.OK)
                {
                    Console.WriteLine("Read time: " + readTime);
                    Console.WriteLine("Gap time: " + gapTime);
                }
                else
                    Console.WriteLine("Error getting read and gap time");
            }
            catch (Exception ex)
            {
                Console.WriteLine("Get Read & Gap Times: " + ex.Message);
            }
        }

    }
}
