﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Xml;
using System.Reflection;
using Microsoft.Win32;
using RFID.Reader;
using RFID.ReaderComm;

namespace ReadWriteTagData
{
    class ReadWriteTagData
    {
        const String DEFAULT_READER_IP_ADDRESS = "192.168.1.120";
        const int PORT_NUMBER = 5000;
        const int maxResponseSize = 1000;
        static RFIDReader reader = null;
        const int TAG_ACCESS_PASSWORD = 0x00000000;
        const int TAG_KILL_PASSWORD = 0x00000000;
        static String[] TAG_EPC = { "66", "66", "66", "66", "66", "66", "66", "66", "66", "66", "66", "66" };

        private static String[] TAG_ID = { "", "", "", "", "", "" };



        static void Main(string[] args)
        {
            try
            {
                String readerIPAddress = DEFAULT_READER_IP_ADDRESS;
                UInt16 PortNumber = PORT_NUMBER;

                Console.WriteLine("Setting up ReaderComm instance on IP address " + readerIPAddress + "\n\n\n");
                Console.WriteLine("Read and Write Tags Test\n" + "///////////////////////////////////////////////////////////////\n");

                reader = new RFIDReader(readerIPAddress, PortNumber);

                ReadTagData();
                UpdateSelectedTagFields();
                ReadTagData2();
                ResetTagData();

            }
            catch (Exception ex)
            {
                Console.WriteLine("Main: " + ex.ToString());
            }
        }

        private static void ReadTagData()
        {
            try
            {
                Byte[] dataBanks = { 0, 0, 1, 2, 3 };
                UInt16[] dataBankLengths = { 2,     // Kill Password
                                             2,     // Access Password
                                             8,     // EPC length
                                             6,     // TID length
                                             8 };   // User data length
                UInt16[] dataBankAddress = { 0, 2, 0, 0, 0 };

                Byte filterBank = 1; // EPC
                UInt16 filterAddress = 32;
                UInt16 filterBitLength = 96;
                Byte[] filterMask = new Byte[12];

                String[] epc = TAG_EPC;
                int bytesInMask = epc.Length;
                if (bytesInMask > filterMask.Length)
                    bytesInMask = filterMask.Length;

                for (int idx = 0; idx < bytesInMask; idx++)
                    filterMask[idx] = Byte.Parse(epc[idx], System.Globalization.NumberStyles.HexNumber);

                for (int idx = 0; idx < dataBanks.Length; idx++)
                {
                    Byte errorCode = 0;

                    UInt16[] data = new UInt16[100];
                    Byte dataBank = dataBanks[idx];
                    UInt16 dataLength = dataBankLengths[idx];

                    String thePassword = "";
                    UInt32 accessPassword = 0x00000000;
                    Boolean dataOk = true;

                    dataBank = dataBanks[idx];

                    if (!dataOk)
                        return;

                    RFIDStatus status = ReadWriteTagData.reader.ReadTagData(accessPassword,
                                                                            filterBank, filterAddress, filterBitLength, filterMask,
                                                                            dataBank, dataBankAddress[idx], dataLength, ref data,
                                                                            out errorCode);
                    if (status == RFIDStatus.OK)
                    {
                        switch (idx)
                        {
                            case 0:
                                thePassword = (data[0] << 16 | data[1]).ToString("X8");
                                Console.WriteLine("Kill Password: " + thePassword);
                                break;

                            case 1:
                                thePassword = (data[0] << 16 | data[1]).ToString("X8");
                                Console.WriteLine("Access Password: " + thePassword);
                                break;

                            case 2:
                                String EPCFull = "";
                                for (int jdx = 2; jdx < dataLength; jdx++)
                                {
                                    String epcWord = data[jdx].ToString("X4");


                                    EPCFull += epcWord + " ";
                                }
                                Console.WriteLine("EPC: " + EPCFull);

                                break;
                            case 3:
                                String tidTag = "";

                                for (int jdx = 0; jdx < dataLength; jdx++)
                                {
                                    String tidWord = data[jdx].ToString("X4");
                                    tidTag += tidWord + " ";
                                    TAG_ID[jdx] = tidWord;

                                }
                                Console.WriteLine("Tag ID: " + tidTag);


                                break;

                            case 4:
                                String userInfo = " ";
                                for (int jdx = 0; jdx < dataLength; jdx++)
                                {
                                    String umWord = data[jdx].ToString("X4");

                                    userInfo += umWord + " ";
                                }
                                Console.WriteLine("User Data: " + userInfo);

                                break;
                        }

                    }
                    else
                        Console.WriteLine("Read Tag Data (" + dataBank.ToString() + ") failed. Error code: " + errorCode.ToString());
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Read Tag Data: " + ex.Message);
            }
        }

        private static void ReadTagData2()
        {
            try
            {
                Byte[] dataBanks = { 0, 0, 1, 2, 3 };
                UInt16[] dataBankLengths = { 2,     // Kill Password
                                             2,     // Access Password
                                             8,     // EPC length
                                             6,     // TID length
                                             8 };   // User data length
                UInt16[] dataBankAddress = { 0, 2, 0, 0, 0 };

                Byte filterBank = 1; // EPC
                UInt16 filterAddress = 32;
                UInt16 filterBitLength = 96;
                Byte[] filterMask = new Byte[12];

                String[] epc = { "66", "66", "66", "66", "66", "66", "66", "66", "66", "66", "55", "55" };
                int bytesInMask = epc.Length;
                if (bytesInMask > filterMask.Length)
                    bytesInMask = filterMask.Length;

                for (int idx = 0; idx < bytesInMask; idx++)
                    filterMask[idx] = Byte.Parse(epc[idx], System.Globalization.NumberStyles.HexNumber);

                for (int idx = 0; idx < dataBanks.Length; idx++)
                {
                    Byte errorCode = 0;

                    UInt16[] data = new UInt16[100];
                    Byte dataBank = dataBanks[idx];
                    UInt16 dataLength = dataBankLengths[idx];

                    String thePassword = "";
                    UInt32 accessPassword = 0x00000000;
                    Boolean dataOk = true;

                    dataBank = dataBanks[idx];

                    if (!dataOk)
                        return;

                    RFIDStatus status = ReadWriteTagData.reader.ReadTagData(accessPassword,
                                                                            filterBank, filterAddress, filterBitLength, filterMask,
                                                                            dataBank, dataBankAddress[idx], dataLength, ref data,
                                                                            out errorCode);
                    if (status == RFIDStatus.OK)
                    {
                        switch (idx)
                        {
                            case 0:
                                thePassword = (data[0] << 16 | data[1]).ToString("X8");
                                Console.WriteLine("Kill Password: " + thePassword);
                                break;

                            case 1:
                                thePassword = (data[0] << 16 | data[1]).ToString("X8");
                                Console.WriteLine("Access Password: " + thePassword);
                                break;

                            case 2:
                                String EPCFull = "";
                                for (int jdx = 2; jdx < dataLength; jdx++)
                                {
                                    String epcWord = data[jdx].ToString("X4");


                                    EPCFull += epcWord + " ";
                                }
                                Console.WriteLine("EPC: " + EPCFull);

                                break;
                            case 3:
                                String tidTag = "";

                                for (int jdx = 0; jdx < dataLength; jdx++)
                                {
                                    String tidWord = data[jdx].ToString("X4");
                                    tidTag += tidWord + " ";
                                    TAG_ID[jdx] = tidWord;

                                }
                                Console.WriteLine("Tag ID: " + tidTag);


                                break;

                            case 4:
                                String userInfo = " ";
                                for (int jdx = 0; jdx < dataLength; jdx++)
                                {
                                    String umWord = data[jdx].ToString("X4");

                                    userInfo += umWord + " ";
                                }
                                Console.WriteLine("User Data: " + userInfo);

                                break;
                        }

                    }
                    else
                        Console.WriteLine("Read Tag Data (" + dataBank.ToString() + ") failed. Error code: " + errorCode.ToString());
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Read Tag Data: " + ex.Message);
            }
        }

        private static Boolean UpdateSelectedTagFields()
        {
            Boolean dataOk = true;

            Byte maskBank = 0x02; // Mask on TID
            UInt16 maskStartAddress = 0x0000;
            UInt16 maskLength = 96; // bits
            Byte[] mask = new Byte[12];

            Byte errorCode = 0;
            UInt32 accessPassword = 0x00000000;
            UInt32 killPassword = TAG_KILL_PASSWORD;
            UInt32 accessPasswordUpdate = TAG_ACCESS_PASSWORD;

            for (int idx = 0; idx < maskLength / 16; idx++)
            {
                UInt16 tidWord = 0;

                tidWord = Convert.ToUInt16(TAG_ID[idx], 16);

                mask[idx * 2] = (Byte)((tidWord >> 8) & 0xFF);
                mask[idx * 2 + 1] = (Byte)(tidWord & 0xFF);
            }


            String[] dataFieldNames = { "Kill Password", "Access Password", "EPC Data", "User Data" };
            Byte[] dataBanks = { 0, 0, 1, 3 };
            UInt16[] dataBankLengths = { 2,     // Kill Password
                                         2,     // Access Password
                                         6,     // EPC length
                                         8 };   // User data length
            UInt16[] dataBankAddresses = { 0, 2, 2, 0 };

            Byte dataBank;
            UInt16 dataBankAddress;
            UInt16 dataLength;
            UInt16[] data = new UInt16[8];
            int attempts = 0;

            for (int fieldIndex = 0; fieldIndex < dataBanks.Length; fieldIndex++)
            {
                dataOk = true;

                dataBank = dataBanks[fieldIndex];
                dataBankAddress = dataBankAddresses[fieldIndex];
                dataLength = dataBankLengths[fieldIndex];

                switch (fieldIndex)
                {
                    case 0:
                        // Updating Kill Password
                        killPassword = killPassword + 1;

                        for (int idx = 0; idx < 2; idx++)
                        {
                            data[idx] = (UInt16)((killPassword >> (1 - idx) * 16) & 0xFFFF);
                        }
                        break;

                    case 1:
                        // Updating Access Password
                        accessPasswordUpdate = accessPasswordUpdate + 1;
                        for (int idx = 0; idx < 2; idx++)
                        {
                            data[idx] = (UInt16)((accessPasswordUpdate >> (1 - idx) * 16) & 0xFFFF);
                        }


                        break;

                    case 2:
                        //Updating EPC data

                        String[] epcParts = { "6666", "6666", "6666", "6666", "6666", "5555" };

                        for (int idx = 0; idx < dataLength; idx++)
                        {

                            UInt16 fldWord = Convert.ToUInt16(epcParts[idx], 16);

                            data[idx] = fldWord;

                        }
                        break;

                    case 3:
                        // Updating User data
                        String[] userData = { "0000", "0000", "0000", "0000", "0000", "0000", "0000", "0006" };
                        for (int idx = 0; idx < dataLength; idx++)
                        {

                            UInt16 fldWord = Convert.ToUInt16(userData[idx], 16);

                            data[idx] = fldWord;
                        }
                        break;
                }


                RFIDStatus status = ReadWriteTagData.reader.WriteTagData(accessPassword,
                                                                         maskBank, maskStartAddress, maskLength, mask,
                                                                         dataBank, dataBankAddress, dataLength, data,
                                                                         out errorCode);
                if(errorCode == 3 || errorCode == 1  && attempts <= 1 )
                {
                    Console.WriteLine("Error updating data retrying ... ");
                    fieldIndex--;
                    attempts++;
                }
            }
            Console.WriteLine("\nTag data written ...\n\n");

                return dataOk;
            }

        private static Boolean ResetTagData()
        {
            Boolean dataOk = true;

            Byte maskBank = 0x02; // Mask on TID
            UInt16 maskStartAddress = 0x0000;
            UInt16 maskLength = 96; // bits
            Byte[] mask = new Byte[12];

            Byte errorCode = 0;
            UInt32 accessPassword = 0x00000000;
            UInt32 killPassword = TAG_KILL_PASSWORD;
            UInt32 accessPasswordUpdate = TAG_ACCESS_PASSWORD;

            for (int idx = 0; idx < maskLength / 16; idx++)
            {
                UInt16 tidWord = 0;

                tidWord = Convert.ToUInt16(TAG_ID[idx], 16);

                mask[idx * 2] = (Byte)((tidWord >> 8) & 0xFF);
                mask[idx * 2 + 1] = (Byte)(tidWord & 0xFF);
            }


            String[] dataFieldNames = { "Kill Password", "Access Password", "EPC Data", "User Data" };
            Byte[] dataBanks = { 0, 0, 1, 3 };
            UInt16[] dataBankLengths = { 2,     // Kill Password
                                         2,     // Access Password
                                         6,     // EPC length
                                         8 };   // User data length
            UInt16[] dataBankAddresses = { 0, 2, 2, 0 };

            Byte dataBank;
            UInt16 dataBankAddress;
            UInt16 dataLength;
            UInt16[] data = new UInt16[8];
            int attempts = 0;

            for (int fieldIndex = 0; fieldIndex < dataBanks.Length; fieldIndex++)
            {
                dataOk = true;

                dataBank = dataBanks[fieldIndex];
                dataBankAddress = dataBankAddresses[fieldIndex];
                dataLength = dataBankLengths[fieldIndex];

                switch (fieldIndex)
                {
                    case 0:
                        // Updating Kill Password

                        for (int idx = 0; idx < 2; idx++)
                        {
                            data[idx] = (UInt16)((killPassword >> (1 - idx) * 16) & 0xFFFF);
                        }
                        break;

                    case 1:
                        // Updating Access Password

                        for (int idx = 0; idx < 2; idx++)
                        {
                            data[idx] = (UInt16)((accessPasswordUpdate >> (1 - idx) * 16) & 0xFFFF);
                        }


                        break;

                    case 2:
                        //Updating EPC data

                        String[] originalEPC = TAG_EPC;

                        String[] epcParts = { originalEPC[0] + originalEPC[1], originalEPC[2] + originalEPC[3],
                                              originalEPC[4] + originalEPC[5], originalEPC[6] + originalEPC[7],
                                              originalEPC[8] + originalEPC[9], originalEPC[10] + originalEPC[11] };

                        for (int idx = 0; idx < dataLength; idx++)
                        {

                            UInt16 fldWord = Convert.ToUInt16(epcParts[idx], 16);

                            data[idx] = fldWord;

                        }
                        break;

                    case 3:
                        // Updating User data
                        String[] userData = { "0000", "0000", "0000", "0000", "0000", "0000", "0000", "0004" };
                        for (int idx = 0; idx < dataLength; idx++)
                        {

                            UInt16 fldWord = Convert.ToUInt16(userData[idx], 16);

                            data[idx] = fldWord;
                        }
                        break;
                }


                RFIDStatus status = ReadWriteTagData.reader.WriteTagData(accessPassword,
                                                                         maskBank, maskStartAddress, maskLength, mask,
                                                                         dataBank, dataBankAddress, dataLength, data,
                                                                         out errorCode);
                if (errorCode == 3 || errorCode == 1 && attempts <= 1)
                {
                    Console.WriteLine("Error updating data retrying ... ");
                    fieldIndex--;
                    attempts++;
                }
            }
            Console.WriteLine("\nTag data Reset ... \n ");

            return dataOk;
        }

    }
    }
